require_relative 'project'
require_relative 'funding_round'
require 'csv'

module Crowdfund
  class Funding
    attr_reader :title

    def initialize(title)
      @title = title
      @projects = []
    end

    def load_projects(from_file)
      CSV.foreach(from_file) do |row|
        project = Project.new(row[0], row[1].to_i, row[2].to_i)
        add_project(project)
      end
    end

    def add_project(a_project)
      @projects << a_project
      puts "#{a_project.name} was added to #{@title}."
    end

    def fund_fun(campaigns)
      @projects.each do |project|
        puts project
      end

      1.upto(campaigns) do |campaign|
          puts "\nCampaign: #{campaign}:"      
          @projects.each do |project|
            FundingRound.spend_or_receive(project)
          end
      end
    end

    def request_funding(campaigns)
      fund_fun(campaigns)
      puts "\n\nProjects:"
      @projects.each do |project|
        puts project
       end
    end

    def funding_partition
      @fully_funded, @under_funded = @projects.partition {|project| project.fully_funded?}
    end

    def funding_status(project)
      formatted_name = project.name.ljust(20,' ')
      "#{formatted_name} \tCurrent Funds: $#{project.funding_pledged} \tStill to Raise: $#{project.funds_still_needed}"
    end

    def save_under_funded_projects(to_file = "needs_funds.txt")
      funding_partition
      sorted_projects = @under_funded.sort
      File.open(to_file, "w") do |file|
        file.puts "\n#{@under_funded.size} #{@title} projects needing funds:"
        sorted_projects.each do |project|
          file.puts funding_status(project)
        end
      end
    end


    def print_stats
      puts "\n#{@title} Statistics:"
      funding_partition
      puts "\n#{@fully_funded.size} fully funded projects:"

      puts "\n#{@under_funded.size} under funded projects:"
      puts "\nPrioritized Projects:"
      sorted_projects = @under_funded.sort
      sorted_projects.each do |project| 
         puts funding_status(project)           
      end
      @projects.sort.each do |project|
        puts "\nProject #{project.name}'s pledges:"
        project.each_pledge_level do |pledge|
          puts "#{pledge.amount} in #{pledge.name} pledges"
        end
      end
    end  
  end
end

if __FILE__ == $0
  puts "=================================" "\n"
  puts "Exercise 10. Fundrequest Class:"

  project1 = Project.new("lmn", 500, 3000)
  puts project1
  project2 = Project.new("xyz", 25, 75)
  puts project2
  project3 = Project.new("abc", 4500, 10000)
  puts project3
  project4 = Project.new("def", 5, 50)
  puts project4
  vc_funds = Funding.new("VC-Friendly Start-up Projects")
  vc_funds.add_project(project1)
  vc_funds.add_project(project2)
  vc_funds.add_project(project3)
  vc_funds.add_project(project4)
  vc_funds.request_funding(2)
end
