require_relative 'fundable'

module Crowdfund
  class Project
    include Fundable
    
    attr_reader :funds, :goal
    attr_accessor :name 
    def initialize(name, funds=0, goal=0)
    	@name = name.upcase
    	@funds = funds
    	@goal = goal
      @pledges = Hash.new(0)
    end

    def received_pledge(pledge)
      @pledges[pledge.name] += pledge.amount

      puts "Project #{@name} received a #{pledge.name} pledge worth $#{pledge.amount}."  
      puts "Project #{name}'s pledges: #{@pledges}"   
    end

    def pledge_amounts
      @pledges.values.reduce(0, :+)
    end
    
    def each_pledge_level
      @pledges.each do |name, amount|
        yield Pledge.new(name, amount)
      end
    end

    def funding_pledged
      @funds + pledge_amounts
    end
    
    def name=(new_name)
      @name=new_name.upcase
    end

    def <=>(other_project)
      other_project.funds_still_needed <=> funds_still_needed
    end

    def to_s
    	"Project #{@name} has $#{funding_pledged} in funds towards a goal of $#{@goal} with $#{funds_still_needed} to be raised."
    end
  end
end

if __FILE__ == $0
  puts "=================================" "\n"
  puts "Exercise 10. Project Class:"
  project1 = Project.new("lmn", 500, 3000)
  puts project1

  project2 = Project.new("xyz", 25, 75)
  puts project2

  project3 = Project.new("abc", 4500, 10000)
  puts project3
  project4 = Project.new("def", 5, 50)
  puts project4
end
